#!/bin/sh
# ex: set expandtab shiftwidth=4 softtabstop=4:

#
#
#  batchretrieve.sh
#
#  tool to get all files based on a certain date range from a DICOM Q/R device.
#
#  Usage:
#  ./batchretrieve.sh (without options) -- interactive mode. 
#
#  In interactive mode, batchretrieve.sh will prompt for the parameters,
#  such as the IP, port, AETITLE, QR model, local AE title, and the date range.
#  If run in interactive mode, it will also create a config file,
#  in ~/var/conf/batchretrieve.rc for defaults
#
#  In non-interactive mode, the parameters are as follows:
#
#  -i Q/R IP address
#  -p Q/R port
#  -a Q/R AETITLE
#  -r Q/R model (root)
#  -t local AETITLE
#  -d date range
#  -l logfile 
#  -q hostid
#
#


. scripttools.new.shi
. runtime_hours.cfg

. $DICOM_VAR/pb-scp.cfg
[ -z "$AE_TITLE" ] && AE_TITLE="$DICOM_AE_TITLE"

LOGDIR="logs"

OK=""
if [ $# = 0 ] ; then

    while [ -z "$OK" ] ; do

        [ -f batchretrieve.rc ] && . batchretrieve.rc

        echo -n "Q/R IP address [$QRIP]:"
        read QRIP1
        [ -n "$QRIP1" ] && QRIP=$QRIP1

        echo -n "Q/R port [$QRPORT]:"
        read QRPORT1
        [ -n "$QRPORT1" ] && QRPORT=$QRPORT1

        echo -n "Q/R AETITLE [$QRAE_TITLE]:"
        read QRAE_TITLE1
        [ -n "$QRAE_TITLE1" ] && QRAE_TITLE=$QRAE_TITLE1

        QRROK=""

        while [ -z "$QRROK" ] ; do
            echo -n "Q/R model ([P]atient/[S]tudy/PatientStudy[O]nly P/S/O) [$QRROOT]:"
            read QRROOT1
            [ -n "$QRROOT1" ] && QRROOT=$QRROOT1
            if [ "$QRROOT" = "P" -o "$QRROOT" = "S" -o "$QRROOT" = "O" ] ; then QRROK=1; fi
        done

        echo -n "local AETITLE [$AE_TITLE]:"
        read AE_TITLE1
        [ -n "$AE_TITLE1" ] && AE_TITLE=$AE_TITLE1

        echo -n "date range (format: 20030125- or 20030125-20030214) [$DATERANGE]:"
        read DATERANGE1
        [ -n "$DATERANGE1" ] && DATERANGE=$DATERANGE1

        echo -n "Debug (y/n):"
        read QRDEBUG1
        QRDEBUG=" -v "
        [ "$QRDEBUG1" = "y" -o "$QRDEBUG1" = "Y" ] && QRDEBUG=" -v -d "

        LOGFILE="br_$DATERANGE.log"
        echo -n "Log file [$LOGFILE]:"
        read LOGFILE1
        [ -n "$LOGFILE1" ] && $LOGFILE=$LOGFILE1


        echo "QRIP=\"$QRIP\""              > $CONF/batchretrieve.rc
        echo "QRPORT=\"$QRPORT\""         >> $CONF/batchretrieve.rc
        echo "QRAE_TITLE=\"$QRAE_TITLE\"" >> $CONF/batchretrieve.rc
        echo "QRROOT=\"$QRROOT\""         >> $CONF/batchretrieve.rc
        echo "AE_TITLE=\"$AE_TITLE\""     >> $CONF/batchretrieve.rc
        echo "DATERANGE=\"$DATERANGE\""   >> $CONF/batchretrieve.rc

        echo  
        echo -n "Is it all correct (Y/N)?"
        read r
        if [ "Y" = "$r" -o "y" = "$r" ] ; then OK=1 ; fi
        echo

    done
else

    QRPORT=104
    QRROOT=P

    [ -f batchretrieve.rc ] && . batchretrieve.rc
    while getopts "i:p:a:r:t:d:l:q:v" OPT ; do
        case $OPT in 
            i) QRIP="$OPTARG" ;;
            p) QRPORT="$OPTARG" ;;
            a) QRAE_TITLE="$OPTARG" ;;
            r) QRROOT="$OPTARG" ;;
            t) AE_TITLE="$OPTARG" ;;
            d) DATERANGE="$OPTARG" ;;
            l) LOGFILE="$OPTARG" ;;
            # Use -q <deviceid> on commandline to use IP, AE, and port from Target table
            q) queryparams=$(echo "select HOST, PORT, AE from Target where ID='$OPTARG';" |\
               ~/component/mysql/bin/mysql -N -B imagemedical| \
               sed 's/\t/ /g') 
               set -- $queryparams
               QRIP=$1
               QRPORT=$2
               QRAE_TITLE=$3
            ;;
            # Use -v on commandline to perform verification only (don't pull pack images, just check count)
            v) VERIFY=1 ;;
        esac
    done

    if [ -z "$LOGFILE" ]; then
        [ ! -d "$LOGDIR" ] && mkdir $LOGDIR
        LOGFILE="$LOGDIR/br_$DATERANGE.log"
    fi

    if [ -z "$QRIP" -o -z "$QRAE_TITLE" ] ; then
        echo "No IP or AE_TITLE given" | tee -a $LOGFILE
        exit
    fi

    #  echo "Q/R IP address = $QRIP"
    #  echo "Q/R port = $QRPORT"
    #  echo "Q/R AETITLE = $QRAE_TITLE"
    #  echo "Q/R root = $QRROOT"
    #  echo "Local AETITLE = $AE_TITLE"
    #  echo "Date range = $DATERANGE"
    #  echo "Log file = $LOGFILE"

    #  exit

fi

#
#  DCM file for find query
#

echo "(0008,0052) CS [STUDY]
(0020,000d) UI [] # Study IUID
(0020,1208) IS [] # Number of Study related images
(0010,0010) PN [] # Patient Name
(0008,0020) DA [] # Study Date
(0008,0030) TM [] # Study Time
(0008,0061) CS [] # Modality (-ies)
" > brf.dmp

~/component/dcmtk/bin/dump2dcm brf.dmp brf.dcm >/dev/null 2>&1
  
#
# DCM file for move
#  

echo "(0008,0052) CS [STUDY]" > br.dmp
~/component/dcmtk/bin/dump2dcm br.dmp br.dcm >/dev/null 2>&1


#
# DCM file for only number of study related images
#
echo "(0008,0052) CS [STUDY]
(0020,000d) UI [] # Study IUID
(0020,1208) IS [] # Number of Study related images
" > q.dmp

$DCMTK_HOME/bin/dump2dcm q.dmp q.dcm >/dev/null 2>&1

echo "(0008,0052) CS [STUDY]
" > s.dmp

$DCMTK_HOME/bin/dump2dcm s.dmp s.dcm >/dev/null 2>&1


#
# Logfile header
#  


echo `date` "Getting studies in the range of $DATERANGE from $QRIP:$QRPORT/$QRAE_TITLE in '$QRROOT' model" | tee -a $LOGFILE


#
# Do a find and process result to log matching studies
#

#  ~/component/dicom/bin/findscu $QRDEBUG -$QRROOT --aetitle $AE_TITLE \
#       --key 0008,0020="${DATERANGE}" --key 0008,0030="000000-" \
#       --call $QRAE_TITLE $QRIP $QRPORT brf.dcm 2>brfinderr.log >brfind.log 
# --key 0008,0030="000000-" doesn't appear to work (0008,0030 is studytime)
~/component/dicom/bin/findscu $QRDEBUG -$QRROOT \
    --aetitle $AE_TITLE \
    --key 0008,0020="${DATERANGE}" \
    --call $QRAE_TITLE \
    $QRIP $QRPORT brf.dcm 2>brfinderr.log >brfind.log


if [ $? != 0 ] ; then
    echo "Error querying archive" | tee -a $LOGFILE
    cat brfinderr.log | tee -a $LOGFILE
    touch /tmp/kill-script.$$
    exit 1
fi

# Wipe out the list of studies to do
[ -e brstyiuid.log ] && rm -f brstyiuid.log

cat brfind.log | awk '
function getdata()
{
  m=index($0,"[");
  if (m!=0)
  {
    r=substr($0,m+1);
    return substr(r,1,index(r,"]")-1)
  }
  else return "-";
}
$1=="RESPONSE:"   { responses=1;}
$1=="(0008,0020)" { date=getdata(); } 
$1=="(0008,0030)" { time=getdata(); }
$1=="(0008,0061)" { mod=getdata(); }
$1=="(0010,0010)" { pname=getdata(); }
$1=="(0020,000d)" { styiuid=getdata(); if (responses==1) {print pname,mod,date,time, styiuid; print styiuid >>"brstyiuid.log"; } }
$1=="(0020,1208)" { a=getdata(); if (responses==1) { obj+=a; stn+=1 } } 
END { print "Total number of studies (objects) to be transferred: " stn " (" obj ")" ; } 
' | tee -a $LOGFILE

rm brfind.log

[ ! -e brstyiuid.log ] && echo "No studies found for ${DATERANGE}" | tee -a $LOGFILE && exit

[ -e brerrors ] && rm -f brerrors

SKIPLIST=skip.ls

cat brstyiuid.log | while read styiuid; do
    if [ -z "$VERIFY" ]; then # only check load if we aren't in verify only mode
        niceme >&2 && touch brerrors && break
    fi
    # Check if I should skip this study
    if grep -w $styiuid $SKIPLIST > /dev/null 2>&1; then
        echo "$styiuid is in $SKIPLIST, skipping"
        continue
    fi
    # Check if I already have this study
    ishere=$(~/component/tools/sql.sh "select styiuid from Dcstudy where styiuid = '$styiuid';" -N)
    if [ -n "$ishere" ]; then
        localobjs=$(~/component/tools/sql.sh "select numofobj from Dcstudy where styiuid = '$styiuid';" -N)
        [ $? -ne 0 ] && echo "Error querying number of local objects, exiting" && echo "$styiuid" >> brerrors && break
        [ -f q.res ] && rm -f q.res q.ls
        ~/component/dicom/bin/findscu $QRDEBUG -$QRROOT \
            --aetitle $AE_TITLE \
            --key 0020,000d="$styiuid" \
            --call $QRAE_TITLE \
            $QRIP $QRPORT q.dcm > q.res
        [ $? -ne 0 ] && echo "Error querying number of objects, exiting" && echo "$styiuid" >> brerrors && break

#        awk '$1=="(0020,000d)"  {s=$3;}
#             $1=="(0020,1208)"  {if (s ~ "^\\[.*\\]$" ) { s= s "\t" $3; 
#             gsub("\\[","",s); gsub("\\]","",s);gsub("\\.00$","",s); print s; } }' q.res > q.ls

        awk '$1=="(0020,000d)"  {s=$3;}
             $1=="(0020,1208)"  {if (s ~ "^\\[.*$" ) { s= s "\t" $3; 
             gsub("\\[","",s); gsub("\\]","",s);gsub("\\.00$","",s); print s; } }' q.res > q.ls
        queriedobjs=$(cat q.ls|cut -f 2)
        echo "$styiuid : ($localobjs/$queriedobjs) (local/queried)"
        if [ "$localobjs" -ge "$queriedobjs" ]; then
            echo "$styiuid is already on this server, skipping"
            continue # cat brstyiuid.log | while read styiuid; do
        else
            echo "$styiuid needs retrieval"
        fi
    else
        echo "$styiuid needs retrieval"
    fi

    if [ -z "$VERIFY" ]; then
        # Do the move
        echo -n "Moving $styiuid..."

        ~/component/dicom/bin/movescu $QRDEBUG -$QRROOT \
            --move $AE_TITLE \
            --aetitle $AE_TITLE \
            --key 0020,000d=$styiuid \
            --call $QRAE_TITLE \
            $QRIP $QRPORT br.dcm 2>brmoverr.log >brmove.log

        if [ $? -ne 0 ] ; then
            echo " movescu_error"
            cat brmoverr.log
            echo "$styiuid" >> brerrors && break
        else
            echo "  OK" 
            # Mark study as viewed
            ./setMigratedStudyStatus.py $styiuid viewed
            if [ $? -ne 0 ]; then
                echo "Error setting $styiuid to viewed"
                echo "$styiuid" >> brerrors && break
            fi
        fi
    fi
done | tee -a $LOGFILE

if [ ! -e brerrors ] ; then
    echo "Done: Successfully moved all studies" | tee -a $LOGFILE
else
    echo "Done at $(date): some errors encountered, see logfile" | tee -a $LOGFILE
    echo "Failed studies:" | tee -a $LOGFILE
    cat brerrors | tee -a $LOGFILE
    cat brerrors >> failedstudies
    touch /tmp/kill-script.$$
    exit 1
fi

echo >> $LOGFILE
echo >> $LOGFILE

