#! /bin/bash


startdate=$1
stopdate=$2

[ "verify" == "$3" ] && verify="-v"

# Uncomment brokenquery if the remote pacs will not accept query dates like 20110808-20110808
# for a one day query. This will cause the script to generate a one day query date like 
# 20110808-20110809 (which might return 20110808's studies)

brokenquery=1

# currentdate.state will contain the last starting date that was queried for
# At the completion of batchretrieve.sh run it will be decremented
# We use it to keep track of what day was done last in case the script is interrupted for some reason.

if [ -f currentdate.state ]; then
    read -p "Resume from last successful date? Y/n: " ans
    if [ "$ans" = "Y" -o "$ans" = "y" ]; then
        . currentdate.state
    else
        rm -f currentdate.state
    fi
fi

[ -z "$currentdate" ] && currentdate=$startdate

#### NOTES
# date --date="20100610 1 day ago" +%Y%m%d
# returns 20100609

# date --date="20100610 1 day" +%Y%m%d
# returns 20100611
#################

while [ ${currentdate} -ge ${stopdate} ]; do
    echo "currentdate=$currentdate" > currentdate.state
    if [ -z "$brokenquery" ]; then
        querydate="${currentdate}-${currentdate}"
    else
        queryenddate=$(date --date="$currentdate 1 day" +%Y%m%d)
        querydate="${currentdate}-${queryenddate}"
    fi
    echo "./batchretrieve.sh -d $querydate"
    ./batchretrieve.sh -d $querydate $verify
    if [ $? -ne 0 ]; then
        echo "batchretrieve.sh returned an error, stopping"
        break
    fi
    nextdate=$(date --date="$currentdate 1 day ago" +%Y%m%d)
    currentdate=$nextdate
done
