#!/bin/bash
######################################################################################################################
#
# checkSentDicomstudies.sh - given full path to a relocated study directory, checks if each object is on the local 
#  	server and if so deletes it from the relocated directory.
# $1 - 	Full path to study: i.e. /home/medsrv/data/dicom.cleanup/1.2.3.4 
#	or
#	/home/medsrv/data/dicom.hashed.cleanup/1Y/UG/ZX/1.2.3.4 
#	or
#      	/home/medsrv/var/archive/archivex/201212/1/1.2.3.4
#	or
#	/home/medsrv/var/archive/archiveZZ/relocated/anotherrelocated/1.2.3.4
#
#	You get the point, that it doesn't matter where the study is located
#	because the FULL path must be provided
######################################################################################################################

# Source the environment setup
. $HOME/etc/setenv.sh

# Source the dicom config file
. $HOME/var/dicom/pb-scp.cfg

# Sending application
STORESCU="$HOME/component/dicom/bin/storescu -R +C"

# Successful check creates a flag file...
FlagFile="AllOnSystem.ok"


if [ -z "$1" ] ; then 
    echo -e "\nERROR: No argument given\n"
    echo -e "\n\tUsage: $0 fullpathtostudydirectory\n\n"
    exit 1
elif [ ! -d "$1" ] ; then
    echo -e "\nERROR: Argument given is not a study directory\n"
    echo -e "\n\tUsage: $0 fullpathtostudydirectory\n\n"
    exit 2
fi

Study=`basename $1`
StudyDir=`$REPOSITORY_HANDLER_HOME/scripts/locateStudy.sh -d $Study`

if [ -z "$StudyDir" ] ; then
   echo -e "\tError: $Study has NO REPOsitory DIRectory.  Exiting."
fi

ReloObjCount=`ls $1  | grep -v $FlagFile | wc -l`
LocalObjCount=`ls $StudyDir | wc -l`
echo -e "\nCurrent Study: $Study"
echo -e "\tLocal Study directory: $StudyDir"
echo -e "\tRelo Study directory: $1"
echo -e "\tRelo Object count: $ReloObjCount"
echo -e "\tLocal Object count: $LocalObjCount"

# Check each Object in the Study
for o in `ls $1`
do
   if [ "$o" = "$FlagFile" ] ; then
     	# This study has already been checked ok, so we skip the flag file
	continue
   fi
   echo -e "\tChecking object: $o"
   LocalObj=`ls $StudyDir/$o`
   LocalobjSize=`du -sb $StudyDir/$o | awk '{ print $1 }'`
   ReloObj=`ls $1/$o`
   ReloobjSize=`du -sb $1/$o | awk '{ print $1 }'`
   ALLPRESENT="true"
   if [ "$LocalObj" = "$ReloObj" ] ; then
	echo -e "\t\tERROR:  Local object and relocated object are the same!  Nothing to do.  Exiting..."
        exit 1
   elif [ -z "$LocalObj" ] ; then
	echo -e "\t\tERROR: Local object, $LocalObj does not exist!!!  Exiting with Failure"
	ALLPRESENT="false"
   elif [ $LocalobjSize -gt 0 ] ; then
	echo -e "\t\tAll checks are good for $o"
        echo -e "\t\tRemoving $ReloObj"
        rm $ReloObj
   else
	ALLPRESENT="false"
   fi
   
done

if [ "$ALLPRESENT" = "true" ] ; then
	echo -e "\tAll objects checked out ok for $Study\n"
        touch $1/$FlagFile
else
	echo -e "\tNOT all objects present locally for $Study"
fi


