#!/bin/bash
#
#################################################################################################
#
#  Author: 	Tony Bowles
#  Date:	June 5th, 2009
#  Title:	scheduledSend.sh
#  Purpose:
#  This script is designed to run a sending program.  The sending program is designed to send
#  a single study to a destination.  The script's job is to only call the sending script 
#  during hours that the site is not operating.  Sending and device selection are handled
#  by the Sending Program listed below as $SNDPROG
#
#################################################################################################

# Check input
if [ ! -e "$1" ] ; then 
	echo "ERROR:  No input file given.  Quitting..."
	exit
fi

# Source needed files
. $HOME/etc/setenv.sh
# we are not worried about the queues for this instance, so I commented out the scripttools.shi
#. $HOME/scripttools.shi

#################################################################################################
# Setup constants
#################################################################################################
SLEEPTIME=600
SNDPROG="./sendDicomtoDevice.sh"

#################################################################################################
# Functions
#################################################################################################
function isOffHours()
{
    #
    # days is an array of strings.  Each element is addressed
    # by a day of the week.  The contents of the string for a
    # given day are the hours of operation or hours during which
    # we do not want the script ( $SNDPROG ) to run.
    #
    #  1 = Monday and 7 = Sunday
    days[1]="06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24"
    days[2]="01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24"
    days[3]="01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24"
    days[4]="01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24"
    days[5]="01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17"
    days[6]="27 99"
    days[7]="29 30"

    Today=`date +%u`
    Hour=`date +%H`
    isHour="FALSE"
    isDay="FALSE"

    for h in ${days[${Today}]}
    do
       if [ "$Hour" = "$h" ] ; then
            isHour="TRUE"
       fi
    done

    if [ "$isHour" = "TRUE" ] ; then
         echo 1
    else
         echo 0
    fi
}

#################################################################################################
# Main
#################################################################################################

for s in `cat $1 `
do
#	niceme && break
	while [ `isOffHours ` = "1" ] 
	do
		echo "During operating hours: `date`. We are sleeping $SLEEPTIME seconds..."
		sleep $SLEEPTIME
	done
	$SNDPROG $s
done
