#!/bin/bash
######################################################################################################################
#
# sendDicomtoMe.sh - given full path to a study directory, calls storescu to the localhost for each of the objects.
# $1 - Full path to study: i.e. /home/medsrv/data/dicom/1.2.3.4 or /home/medsrv/data/dicom.hashed/1Y/UG/ZX/1.2.3.4 or
#      even /home/medsrv/var/archive/archivex/201212/1/1.2.3.4
#
# These are example entries of the pb-scp.cfg file
#
# DEFAULT_SEND_TIMEOUT="30"
# PORT="104"
# CASE_INSENSITIVE="true"
# PDU_SIZE="65536"
# AE_TITLE="hub3"
# TLS_PORT="2762"
# MAX_ASSOCIATIONS="16"
# EMPTY_MATCHING="false"
# PROMISCUOUS_MODE="true"
# TRANSFER_SYNTAX="xe"
# STORESCU_FLAG="-R"
# DEFAULT_RECEIVE_TIMEOUT="30"
#
# This is an example entry from the database
#
# ID      	HOST    	HOSTNAME        PORT    AE      APPLICID        FACILITYID      TYPE    PROTOCOL        RECVTO  SENDTO
# LVUROChicago  70.232.9.53     70.232.9.53     2762    PBCHICAGO                       	D       S       0       0

######################################################################################################################

# Source the environment setup
. $HOME/etc/setenv.sh

# Source the dicom config file
. $HOME/var/dicom/pb-scp.cfg

######################################################################################################################
# Setup Constants
######################################################################################################################
CALLEDAET="PBCHICAGO"
PORT="2762"
IP="70.232.9.53"
PROTOCOL="S"  # S = TLS  T = TCP
CALLINGAET="$AE_TITLE"


######################################################################################################################
# Main 
######################################################################################################################
# Sending application
    STORESCU="$HOME/component/dicom/bin/storescu -R +C"

if [ "$PROTOCOL" = "T" ] ; then
    SECURE="OFF"
elif [ "$PROTOCOL" = "S" ] ; then
    SECURE="ON"
else
    echo -e "\n\nERROR: PROTOCOL is not set to either T (TCP) or S (TLS).  Please ensure PROTOCOL is either T or S"
    exit
fi
# Successful check creates a flag file...
FlagFile="AllOnSystem.ok"


if [ -z "$1" ] ; then 
    echo -e "\nERROR: No argument given\n"
    echo -e "\n\tUsage: $0 fullpathtostudydirectory\n\n"
    exit 1
elif [ ! -d "$1" ] ; then
    echo -e "\nERROR: Argument given is not a study directory\n"
    echo -e "\n\tUsage: $0 fullpathtostudydirectory\n\n"
    exit 2
fi

echo -e "\n`date`\tCurrent Study: `basename $1`"

for o in `ls $1`
do
   if [ "$o" = "$FlagFile" ] ; then
      continue
   fi
   echo -e "\tSending object: $o"
   if [ "$SECURE" = "OFF"  ] ; then
       $STORESCU  -aet $CALLINGAET -aec $CALLEDAET $IP $PORT $1/$o
   else
      $OPENSSL_PASS | $HOME/component/dicom/bin/storescu  -v -d -xe -R +C +tls $SSL_SITE_KEY $SSL_SITE_CERT -ic +ps -aet $CALLINGAET -aec $CALLEDAET $IP $PORT $1/$o
   fi
done

