#!/bin/bash
######################################################################################################################
#
# sendDicomtoMe.sh - given full path to a study directory, calls storescu to the localhost for each of the objects.
# $1 - Full path to study: i.e. /home/medsrv/data/dicom/1.2.3.4 or /home/medsrv/data/dicom.hashed/1Y/UG/ZX/1.2.3.4 or
#      even /home/medsrv/var/archive/archivex/201212/1/1.2.3.4
#
# These are example entries of the pb-scp.cfg file
#
# DEFAULT_SEND_TIMEOUT="30"
# PORT="104"
# CASE_INSENSITIVE="true"
# PDU_SIZE="65536"
# AE_TITLE="hub3"
# TLS_PORT="2762"
# MAX_ASSOCIATIONS="16"
# EMPTY_MATCHING="false"
# PROMISCUOUS_MODE="true"
# TRANSFER_SYNTAX="xe"
# STORESCU_FLAG="-R"
# DEFAULT_RECEIVE_TIMEOUT="30"
#
######################################################################################################################

# Source the environment setup
. $HOME/etc/setenv.sh

# Source the dicom config file
. $HOME/var/dicom/pb-scp.cfg

# Sending application
STORESCU="$HOME/component/dicom/bin/storescu -R +C"

# Successful check creates a flag file...
FlagFile="AllOnSystem.ok"


if [ -z "$1" ] ; then 
    echo -e "\nERROR: No argument given\n"
    echo -e "\n\tUsage: $0 fullpathtostudydirectory\n\n"
    exit 1
elif [ ! -d "$1" ] ; then
    echo -e "\nERROR: Argument given is not a study directory\n"
    echo -e "\n\tUsage: $0 fullpathtostudydirectory\n\n"
    exit 2
fi

echo -e "\n`date`\tCurrent Study: `basename $1`"

for o in `ls $1`
do
   if [ "$o" = "$FlagFile" ] ; then
      continue
   fi
   echo -e "\tSending object: $o"
   $STORESCU  -aet $AE_TITLE -aec $AE_TITLE 127.0.0.1 $PORT $1/$o
done

