#! /bin/bash

styiuid=$1

function getCurrentRepository() {
    # Echoes the repository path (including styiuid) or an empty string
    # if the study does not exist in an archive

    local styiuid=$1
    echo  "$(/home/medsrv/component/repositoryhandler/scripts/locateStudy.sh -d $styiuid)"

}


function getArchDir() {
    local styiuid=$1
    local archdir=$(~/component/tools/sql.sh "select archdir from Archive where styiuid = '$styiuid';" -N)
    [ $? -ne 0 ] && { echo "Error querying archdir" >&2  ; return 1; }
    [ -z "${archdir}" ] && { echo "Error: no archdir returned for $styiuid" >&2 ; return 1; }
    echo $archdir
    return 0
} # getArchDir


function getFullArchivePath() {
    local styiuid=$1
    local archdir=$(getArchDir $styiuid)
    [ -n "${archdir}" ] && { echo "/home/medsrv/var/archive/${archdir}/${styiuid}/" ; return 0; } || return 1 
} # getFullArchivePath


function listAllArchivedObjects() {
    # Lists all objects in a study's archive dir
    # If 2nd argument is given, echo number of objects in archive dir instead
    local styiuid=$1
    local count=$2

    [ -z "$count" ] && ls -1 $(getFullArchivePath $styiuid) || ls -1 $(getFullArchivePath $styiuid)|wc -l
}

function listAllRepositoryObjects() {
    local styiuid=$1
    local count=$2

    local repodir=$(getCurrentRepository $styiuid)
    [ -z "$count" ] && ls -1 ${repodir}/ || ls -1 ${repodir}/| wc -l
}

echo "$styiuid (repo) : $(listAllRepositoryObjects $styiuid count) objects"
echo "$styiuid (archive) : $(listAllArchivedObjects $styiuid count) objects"

archivepath=$(getFullArchivePath $styiuid)
repopath=$(getCurrentRepository $styiuid)


for o in $(listAllArchivedObjects $styiuid); do
    echo "*************************************"
    echo "$styiuid/$o:"
    if [ -e "${repopath}/$o" ]; then
        ~/component/dicom/bin/dcmdump ${archivepath}/$o > archive.dump
        ~/component/dicom/bin/dcmdump ${repopath}/$o > repo.dump
        diff archive.dump repo.dump
        [ $? -eq 0 ] && echo "Archive and repo objects are the same"
        echo
    else 
        echo "$o exists in archive but doesn't exist in $repopath"
        echo
    fi
    echo "*************************************"
done


       
