#! /bin/bash
# criley

styiuid=$1

. ~/etc/setenv.sh

usage() {
    echo "usage: $0 styiuid"
    exit
}

[ -z "$styiuid" ] && echo "No styiuid specified - exiting" && usage

SELF_SERVER_ID=""
[ -f "$CONF/serverid.cfg" ] && . $CONF/serverid.cfg
if [ -z "$SELF_SERVER_ID" ] ; then
	echo "Self server id is not set in $CONF/serverid.cfg"
	exit 1
fi


OBSOLETEOBJECTLIST="/home/medsrv/tmp/obsoleteobjectlist"

rm -f $OBSOLETEOBJECTLIST 2>/dev/null

function checkMigrationrepoUsage() {
    local repo=$1

    repousage=$(df -h --sync $repo | tail -1 |awk '{print $4}'|sed 's/\%//')
    echo "Usage on $repo : $repousage / $MIGRATIONREPOFULLLIMIT"
    if [ "$repousage" -lt "$MIGRATIONREPOFULLLIMIT" ]; then
        echo "Under migration repo fullness limit, continue processing"
        return 0
    else
        echo "Not below  migration repo fullness limit, exiting"
        return 1
    fi
    
}

function selectRepository() {
    # echoes a repository that is under MIGRATIONREPOFULLLIMIT and that is in MIGRATIONREPOSITORYLIST
    # and returns 0
    # echoes "NOSUITABLEREPOSITORY" and returns 1 if no repo meets the criteria
    # The caller must check the return code and handle exiting if no repo is found!
    local repo
    local fullpathtorepo
    [ -z "$MIGRATIONREPOSITORYLIST" ] && echo "NOSUITABLEREPOSITORY" && return 1
    for repo in $MIGRATIONREPOSITORYLIST; do
        fullpathtorepo="/home/medsrv/data/dicom.repository/${repo}/"
        if checkMigrationrepoUsage $fullpathtorepo >/dev/null 2>&1; then
            echo "$fullpathtorepo"
            return 0
        fi
    done
    # If we get here, there's not one in the list under the limit
    echo "NOSUITABLEREPOSITORY"
    return 1
}


function getCurrentRepository() {
    # Echoes the repository path (including styiuid) or an empty string
    # if the study does not exist in an archive

    local styiuid=$1
    echo  "$(/home/medsrv/component/repositoryhandler/scripts/locateStudy.sh -d $styiuid)"

}


function getArchDir() {
    local styiuid=$1
    local archdir=$(~/component/tools/sql.sh "select archdir from Archive where styiuid = '$styiuid';" -N)
    [ $? -ne 0 ] && { echo "Error querying archdir" >&2  ; return 1; }
    [ -z "${archdir}" ] && { echo "Error: no archdir returned for $styiuid" >&2 ; return 1; }
    echo $archdir
    return 0
} # getArchDir


function getFullArchivePath() {
    local styiuid=$1
    local archdir=$(getArchDir $styiuid)
    [ -n "${archdir}" ] && { echo "/home/medsrv/var/archive/${archdir}/${styiuid}/" ; return 0; } || return 1 
} # getFullArchivePath


function getHashDir() {
    local styiuid=$1
    echo "$(/home/medsrv/component/utils/bin/getHashDirectory -i $styiuid)"
    return $?
} # getHashDir()


function getObsoleteState() {
    # Returns 0 if a study is wholly or partially obsolete, 1 if not obsolete
    # echoes "WHOLE" if whole, PARTIAL if partial, NOTOBSOLETE if not
    # wholly or partially obsolete

    local styiuid=$1
    local archst=$(~/component/tools/sql.sh "select archst from Archive where styiuid = '$styiuid';" -N)
    local obsoletefile="$(getFullArchivePath ${styiuid})/.info/obsoleted.rec"
    local WHOLE=""
    local FILES=""
    
    if [ "${archst}" -eq -100 ]; then
        echo "WHOLE"
        return 0
    fi    
    if [ -f "${obsoletefile}" ]; then
        . $obsoletefile
        [ -n "$WHOLE" ] && { echo "WHOLE" ; return 0; }
        [ -n "$FILES" ] && { echo "PARTIAL" ; return 0; }
    else
        echo "NOTOBSOLETE"
        return 1
    fi
} #isObsolete()


function listObsoletedObjects() {
    # Lists all objects in a study's that are flagged as obsolete and deletes them
    # (space separated list on stdout AND creates a file one object per line in ~/tmp/obsoleteobjectlist.$styiuid)
    # Assumes that the obsoleted state is partial; call getObsoleteState before calling this function to check.
    # If 2nd argument is given, echo number of obsoleted objects instead (does not delete them in this case)

    local styiuid=$1
    local count=$2

    local obsoletefile="$(getFullArchivePath ${styiuid}).info/obsoleted.rec"
    local WHOLE=""
    local FILES=""


    if [ -f "${obsoletefile}" ]; then
        . $obsoletefile
        if [ -n "$count" ]; then
            filesarray=( $FILES ) # Dirty hack to count number of elements in list
	    echo "${#filesarray[@]}"
            return 0
        else
            if [ -n "$FILES" ]; then
                echo "Files that are obsoleted for this study and will be removed: $FILES"
                for f in $FILES; do
                    echo "$f" >> ${OBSOLETEOBJECTLIST}.${styiuid}
		    rm -f $(getFullArchivePath ${styiuid})${f}
                done
                return 0
            fi
        fi 
    else
        echo "No obsoleted.rec"
        return 1
    fi
}


function listAllArchivedObjects() {
    # Lists all objects in a study's archive dir
    # If 2nd argument is given, echo number of objects in archive dir instead
    local styiuid=$1
    local count=$2

    [ -z "$count" ] && ls -1 $(getFullArchivePath $styiuid) || ls -1 $(getFullArchivePath $styiuid)|wc -l
}


function removeArchiveTableEntry() {
    local styiuid=$1
    ~/component/tools/sql.sh "delete from Archive where styiuid = '$styiuid';"
    [ $? -ne 0 ] && { echo "Error removing archive table entry" ; return 1; } || return 0
} # removeArchiveTableEntry


function removeStudyFromArchive() {
    local styiuid=$1

    ./checkAndRemoveArchivedStudy.sh $(getFullArchivePath $styiuid)

} # removeStudyFromArchive


#function makeNewHashDir() {
#    # DO NOT USE
#    # Creates a hash directory, echoes full repo path on success

#    local styiuid=$1
#    local repo=$2  # full path to repo directory with trailing slash
#    local sdir=`$HOME/component/repositoryhandler/scripts/locateStudy.sh -d $styiuid`

#   # echo "makeNewHashDir: sdir = $sdir"

#   if [ -d "$sdir" ] ; then
#	#3echo "makeNewHashDir: Repo directory already exists: $sdir"
#	newhashdir="$sdir"
#	echo "$newhashdir"
#        return 0
#    fi
#    
#    [ -z "${styiuid}" ] && echo "makeNewHashDir: No styiuid specified">2 && exit 1
#    [ -z "${repo}" ] && echo "makeNewHashDir: No repository specified">2 && exit 1
#    [ ! -d "${repo}" ] && echo "makeNewHashDir: Repository $repo does not exist">2 && exit 1
#    repolastcharpos=$(expr ${#repo} - 1 )
#    repolastchar=${repo:repolastcharpos}
#    if [ "${repolastchar}" != "/" ]; then
#        # if no trailing slash, add it
#        repo="${repo}/"
#    fi


#    local newhashdir="${repo}$(getHashDir ${styiuid})"
#    mkdir -p $newhashdir
#    [ $? -ne 0 ] && echo "nameNewHashDir: error creating hash directory $newhashdir">&2  && exit 1 
#    echo "$newhashdir"
#} # makeNewHashDir

function copyObjectsToHashDir() {
    # Call this on an unobsoleted study to copy all objects from archive to repo

    local styiuid=$1
    local hashdir=$2 
    local sizeinblocks 

    local archivepath=$(getFullArchivePath $styiuid)

    if [ ! -d "$archivepath" ] ; then
	echo "copyObjectsToHashdir: Archive path $archivepath does not exist!!" 
	return 1
    fi

    #check for 0 length files in archive
    for f in $(listAllArchivedObjects $styiuid); do
        sizeinblocks=$(ls -s ${archivepath}/${f} | awk '{print $1}')
        if [ "$sizeinblocks" -eq 0 ]; then
            echo "copyObjectsToHashdir: ${archivepath}/${f} is 0 length, aborting"
            return 2
        fi
    done

    for f in $(listAllArchivedObjects $styiuid); do
        cp -va ${archivepath}/${f} ${hashdir}/
        [ $? -ne 0 ] && { echo "copyObjectsToHashdir: Error copying to $hashdir" ; return 1; }
    done
    return 0
}


function createStorestate() {
    # Creates a storestate with ProcessMode=Store and correct owner and staged device
    # todo... need to deal with adding the correct device id for a wl server if one exists
    # --cwr 
    local styiuid=$1

#    ~/component/utils/bin/storestate --processmode=Store --study=${styiuid} --owner=yes --setstaged wl1
    ProcessMode="Store"
    [ -f $CONF/self.rec ] && . $CONF/self.rec

    # Set ArchiveDevice and SourceDevice attributes and reset ProcessMode
    $UTILS_HOME/bin/storestate --study "$styiuid" --source "@Archive-$SELF_SERVER_ID" --owner=yes --archive "@Archive-$SELF_SERVER_ID" \
		--processmode "$ProcessMode" --retrieved=yes

    if [ $? -ne 0 ]; then
        echo "createStorestate: Error creating storestate.rec"
        return 1
    else
        return 0
    fi
}

function setProcessmodeProcess() {
    # Sets processmode to process
    # Call createStorestate before calling this function
    local styiuid=$1


    ~/component/utils/bin/storestate --processmode=Process --study=${styiuid}
    [ $? -ne 0 ] && { echo "createStorestate: Error setting processmode" ; return 1; } || return 0
}

function setTimeStamp() {
    # Sets the timestamp on the given directory to the date and time of the study
    local styiuid=$1 
    local repositorydir=$2 # full path including styiuid

    local thedate=$(~/component/tools/sql.sh "select thedate from Dcstudy where styiuid = '$styiuid';" -N)
    local thetime=$(~/component/tools/sql.sh "select thetime from Dcstudy where styiuid = '$styiuid';" -N)

    while [ -z "${thedate}" ]; do
        echo "Study is not registered yet, sleeping 5" >&2
	sleep 5
        local thedate=$(~/component/tools/sql.sh "select thedate from Dcstudy where styiuid = '$styiuid';" -N)
        local thetime=$(~/component/tools/sql.sh "select thetime from Dcstudy where styiuid = '$styiuid';" -N)
    done

    sleep 5 # Wait for queue to settle

    numcqtasks=$(find ~/var/cpuqueue |xargs grep -l -w $styiuid | wc -l)
    while [ $numcqtasks -ne 0 ]; do
        sleep 5
    done

    local timestamp="${thedate}${thetime}"
    # truncate to the first 12 digits
    timestamp=${timestamp:0:12}
    touch --no-create -t $timestamp $repositorydir
    [ $? -ne 0 ] && return 1 || return 0
}

function doMigration() {
    local styiuid=$1
    local hashdir

    # Set the storestate first.  This will create a repo directory if one doesn't exist.
    # It sets the archive device, processmode , and study owner. Also retrieved=yes
    # Need to add code to check for a parent (wl) and set it to staged
    # and test that reindexing always updates the wl server

    echo "Create (or fix) storestate for $styiuid"
    createStorestate $styiuid
    [ $? -ne 0 ] && return 1

    # Not trying to force a repo anymore.  Also fixed to call createDirectory instead of making a hash directory manually
    echo "Get/Create repository location"
    hashdir=$($REPOSITORY_HANDLER_HOME/bin/createDirectory $REPOSITORY_DATA_DICOM $styiuid)
    [ $? -ne 0 ] && { echo "Error getting/creating repository location, exiting" ; return 1; }

    copyObjectsToHashDir $styiuid $hashdir 
    exitstatus=$?
    if [ $exitstatus -eq 2 ]; then
        echo "Error copyingobjects to repo (0 length), continuing"
        return 2
    elif [ $exitstatus -ne 0 ]; then
        echo "Error copyingobjects to repo, exiting"
        return 1
    fi

    # Insert preprocessing here

    echo "Reindexing $styiuid ( -e -d )"
    /home/medsrv/component/cases/reindexStudy.sh -e -d $styiuid
    [ $? -ne 0 ] && { echo "Error reindexing $styiuid" ; return 1; }

    # insert postprocessing here
    if [ -e ./migrationpostprocess.sh ]; then
       ./migrationpostprocess.sh $styiuid
       [ $? -ne 0 ] && { echo "Error during postprocessing $styiuid" ; return 1; }
    fi

    echo "remove it from archive"
    removeStudyFromArchive $styiuid
    [ $? -ne 0 ] && { echo "Error removing from archive, exiting" ; return 1; }

#    echo "Setting timestamp on hashdir the same as study date/time"
#    setTimeStamp $styiuid $hashdir
#    [ $? -ne 0 ] && { echo "Error setting timestamp on repo, exiting" ; return 1; }

    return 0
}


##### Main #######

#migrationrepo=$(selectRepository)
#if [ $? -ne 0 ]; then
#    echo "No repository under MIGRATIONREPOFULLLIMIT ($MIGRATIONREPOFULLLIMIT)"
#    echo "MIGRATIONREPOSITORYLIST (defined in migration.cfg) contains:"
#    echo "$MIGRATIONREPOSITORYLIST"
#    exit 1
#else
#    echo "migrationrepo : $migrationrepo"
#fi

currentrepo=$(getCurrentRepository $styiuid)
if [ -n "${currentrepo}" ]; then
    echo "Current repository: $currentrepo"
else
    echo "$styiuid current repository: none"
fi


fullarchivepath=$(getFullArchivePath $styiuid)
if [ $? -ne 0 ]; then
    echo "Error getting the archive path for $styiuid, exiting."
    exit 1
elif [ -z "${fullarchivepath}" ]; then
    echo "$styiuid is not in the Archive table? Nothing to do."
    exit 0
elif [ ! -d "${fullarchivepath}" ]; then
    echo "$fullarchivepath is not accessible, check that the archive partitions are mounted."
    exit 1 
fi

obsoletestate=$(getObsoleteState $styiuid)
case $obsoletestate in  
    WHOLE)
        echo "$styiuid is wholly obsolete"
        echo "Removing $styiuid from archive and archive table"
        rm -rf $fullarchivepath # Set above
        removeArchiveTableEntry $styiuid
        exitstatus=$?
        ;;

    NOTOBSOLETE)
        echo "$styiuid is neither wholly nor partially obsolete"
        echo "Migrate $styiuid"

        doMigration $styiuid
        exitstatus=$?
        ;;

    PARTIAL)
        echo "$styiuid is partially obsolete"

        echo "All objects in study :($(listAllArchivedObjects $styiuid count))"
        listAllArchivedObjects $styiuid
        echo "Obsoleted objects in study :($(listObsoletedObjects $styiuid count))"

        echo "Remove obsolete objects from archive"
        listObsoletedObjects $styiuid 
        [ $? -ne 0 ] && { echo "Error deleting obsolete objects from Archive location" ; exit 1; }

        #cat ${OBSOLETEOBJECTLIST}.${styiuid} # file gets created in ~/tmp by listObsoleteObjects() 
        echo "Migrating non-obsolete objects : ($(listAllArchivedObjects $styiuid count))"

        doMigration $styiuid
        exitstatus=$?
        ;;
    *)
        echo "Unknown obsolete state"
        exitstatus=1
        ;;
esac
[ $exitstatus -ne 0 ] && exit $exitstatus || exit 0
