#! /bin/bash

. ~/etc/setenv.sh

STUDYLS="$$.ls"
STORESTATEVARS="OwnerDevice ProcessMode SourceDevice ArchiveDevice StagedDevices StageState StudyOwner Version ForwardDevices Retrieved"
PBv52=$(head -1 ~/etc/component.ls|awk '{print $4}'|egrep ^5\.2)
REFTYPES="OBJECTFILE STUDYDIR STUDYDB"

hasHash() {
    [ -d "$HOME/data/dicom.hashed" -o -d "$HOME/dicom.repository" ] && return 0 || return 1
}


usage() {

    echo
    echo "Usage: `basename $0` -h -S -R [ -s <styiuid> | -f <file> | -a ]"
    echo
    echo "     -a         Operate on all studies in Dcstudy"
    echo
    echo "     -A         Operate on all studies in ~/data/dicom"
    echo
    echo "     -s         Operate on <styiuid>"
    echo
    echo "     -f         Operate on all styiuids listed in <file>"
    echo "                <file> should contain styiuids, one to a line"
    echo
    echo "     -S         Print storestate.rec"
    echo
    echo "     -R         Print references"
    echo 
    echo "     -o         Print all on one line (better for piping to another script"
    echo "                (This only affects references, not storestate)"
    echo 
    echo "     -h         Display help"
    exit 0
}

parseArgs() {

[ -z "$1" ] && usage

while getopts ":Aacf:s:SRho" OPT ; do
    case $OPT in
        A) if [ -n "$styiuid" -o -n "$styiuidfile" -o -n "$allstudies" ]; then
               echo;echo "-f , -s , -A, and -a are mutually exclusive"
               usage
           fi
           alldatadicom="1"
           ;;
        a) if [ -n "$styiuid" -o -n "$styiuidfile" -o -n "$alldatadicom" ]; then
               echo;echo "-f , -s , -A, and -a are mutually exclusive"
               usage
           fi
           allstudies="1"
           ;;
        c) clean="1" ;; # Use this at your peril!  Deleting all references *will* have the effect of removing the study!
        R) printreferences="1" ;;
        o) onelineoutput="1" ;;
        S) printstorestate="1" ;;
        f) if [ -n "$styiuid" -o -n "$allstudies" -o -n "$alldatadicom" ]; then
               echo ;echo"-f , -s , and -a are mutually exclusive"
               usage
           fi
           styiuidfile="$OPTARG"
           ;;
        s) if [ -n "$styiuidfile" -o -n "$allstudies" -o -n "$alldatadicom" ]; then
               echo; echo "-f , -s , and -a are mutually exclusive"
               usage
           fi
           styiuid="$OPTARG"
           ;;
        h) usage ;;
        *) echo "Unknown option"
           usage
           ;;
    esac
done

}



setStudylist() {

    if [ -n "$styiuidfile" ]; then
        STUDYLS="$styiuidfile"
    elif [ -n "$allstudies" ]; then
        echo "select styiuid from Dcstudy;"|~/component/mysql/bin/mysql -N imagemedical > $STUDYLS 
    elif [ -n "$alldatadicom" ]; then
        ls -1 ~/data/dicom/ > $STUDYLS
    else
        echo $styiuid > $STUDYLS
    fi

    if [ -f "$STUDYLS" ]; then
        echo $STUDYLS
    else
        echo "Error: $STUDYLS: File not found"
        exit 1
    fi 

}

printFlags() {
    for flag in allstudies printreferences printstorestate styiuidfile styiuid studylist clean; do
        [ -n "${!flag}" ] && echo "$flag: ${!flag}"
    done
}


printReferences() {
    local referencelist=$1
    local indentnum=$2
    local reftype=$3
    local styiuid=$4
    local object=$5
    local indent
    local ref
    local space

    if [ -z "$onelineoutput" ]; then
	for space in $(seq 1 $indentnum); do
	    indent="$indent $(echo -n " ")"
	done

	echo
	for ref in $referencelist; do
            [ -z "$ref" ] && ref="NoReferenceFound"
	    echo "${indent}${ref}"
	done
    else
        for ref in $referencelist; do
            [ -z "$ref" ] && ref="NoReferenceFound"
            echo "$styiuid:$object:$reftype:$ref"
        done
    fi
}

hasDicomdir() {
    local styiuid=$1

	repoDir=`$HOME/component/repositoryhandler/scripts/locateStudy.sh -d $styiuid`

    if ! ls -al $repoDir > /dev/null 2>&1 ; then
        return 1
    fi

    return 0
}

printStorestate() {
    local styiuid=$1
    local indentnum=$2
    local tag
    local space
    local indent

	repoDir=`$HOME/component/repositoryhandler/scripts/locateStudy.sh -d $styiuid`

    for space in $(seq 1 $indentnum); do
        indent="$indent $(echo -n " ")"
    done

    echo "${indent}storestate.rec:"

    unset indent
    (( indentnum += 4 ))
    for space in $(seq 1 $indentnum); do
        indent="$indent $(echo -n " ")"
    done


    if hasDicomdir $styiuid; then
        # unset possible storestate variables here!
        for tag in $STORESTATEVARS; do
            unset $tag
        done

        . $repoDir/.info/storestate.rec

        for tag in $STORESTATEVARS; do
            [ -n "${!tag}" ] && echo "${indent}${tag} : ${!tag}"
        done
    else
        echo "${indent}No ~/data/dicom directory!"
    fi
}



checkRefs() {

    local styiuid=$1
    local reftype=$2
    local res
    local refcountargs
    local objectlist
    local object
	repoDir=`$HOME/component/repositoryhandler/scripts/locateStudy.sh -d $styiuid`

    [ -z "$onelineoutput" ] && echo -n "    Checking $reftype refs : "

    if [ "$reftype" == "OBJECTFILE" ]; then
        if [ -z "$repoDir" ] ; then
            echo "No Repository dir"
            return
        else
            objectlist=$(ls -1 $repoDir/)
            if [ -z "$objectlist" ]; then
                echo "No objects"
                return
            else
                for object in $objectlist; do
                    [ -z "$onelineoutput" ] && echo -en "\n        $object : "
                    #if hasHash; then
                        res=$(~/component/utils/bin/refcount -p -i $styiuid/$object -t $reftype 2>/dev/null)
                        if [ -n "$clean" ]; then
                            for reference in $res; do
                                ~/component/utils/bin/refcount -r $reference -i $styiuid/$object -t $reftype
                            done
                        fi
                    #else
                    #    res=$(cat ~/var/refcount/OBJECTFILE/$styiuid/$object)
			#if [ -n "$clean" -a -n "$res" ]; then
			#    unset TYPE
			#    unset REFERRER
			#    for reference in $res; do
		#		eval $reference
		#		if [ -n "$TYPE" -a -n "$REFERRER" ]; then
		#		    ~/component/utils/bin/refcount -r $REFERRER -i $styiuid/$object -t $TYPE
		#		fi
		#	    done
		#	fi
                #    fi
                    printReferences "$res" 6 $reftype $styiuid $object
                done
            fi
        fi
    else
#        if hasHash; then
            res=$(~/component/utils/bin/refcount -p -i $styiuid -t $reftype 2>/dev/null)
	    if [ -n "$clean" ]; then
		for reference in $res; do
		    echo -e "\n ~/component/utils/bin/refcount -r $reference -i $styiuid -t $reftype\n"
		    ~/component/utils/bin/refcount -r $reference -i $styiuid -t $reftype
		done
	    fi
#        else
#            res=$(cat ~/var/refcount/$reftype/$styiuid)
#	    if [ -n "$clean" -a -n "$res" ]; then
#		unset TYPE
#		unset REFERRER
#		for reference in $res; do
#		    eval $reference
#		    if [ -n "$TYPE" -a -n "$REFERRER" ]; then
#			~/component/utils/bin/refcount -r $REFERRER -i $styiuid -t $TYPE
#		    fi
#		done
#	    fi
#
#        fi
        printReferences "$res" 4 $reftype $styiuid $object
    fi
}

    
# Main

parseArgs "$@"
studylist=$(setStudylist)
printFlags

while read styiuid; do 
    echo "#################"
    echo "$styiuid"
    if [ -n "$printreferences" ]; then
        for reftype in $REFTYPES; do
            checkRefs $styiuid $reftype
        done
    fi
    echo
    [ -n "$printstorestate" ] && printStorestate $styiuid 2
    echo;echo
done < $studylist
[ -f "$$.ls" ] && rm $$.ls

