#! /bin/bash
#
#

. ~/etc/setenv.sh
. $DICOM_VAR/pb-scp.cfg

_promptluser(){
  
    local emessage

    if [ -z "$1" ]; then
        emessage="Option parse error: -$OPT : requires parameter"
    elif [ "${1:0:1}" == "-" ]; then
        emessage="Option parse error: -$OPT $1 : did you forget to supply a parameter to -$OPT?"
    fi
    if [ -n "$emessage" ]; then
        echo;echo;echo
        echo "$0 ERROR"
        echo "$emessage"
        $0 -h
        echo;echo;echo
        exit 1
    fi
} # End _promptluser

while getopts ":hq:s:T:t:" OPT ; do
    case $OPT in
        h) echo
           echo "Usage: `basename $0` -hq:s:T:t:"
           echo
           echo "     -h                    Display help"
           echo "     -q <hostid>           Query number of study related objects from <hostid>"
           echo "     -t <seconds>          Set SEND timeout to <seconds>"
           echo "     -T <seconds>          Set RECEIVE timeout to <seconds>"
           echo "     -s <styiuid>          query for study uid <styiuid>"
           exit 0
           ;;


        q) query="$OPTARG"
           _promptluser $query
           ;;


        s) styiuid="$OPTARG"
           _promptluser $styiuid
           ;;

        t) STO="$OPTARG"
           _promptluser $STO
           ;;

        T) RTO="$OPTARG"
           _promptluser $RTO
           ;;


        *) echo "No such option $OPTARG"
           $0 -h
           exit 0
           ;;

    esac
done
[ -z "$query" ] && echo "$0 : Nothing to do (try the -q option, or -h for full list)" && exit 0

if [ -n "$query" ]; then
    queryparams=$(echo "select HOST, PORT, AE from Target where ID='$query';" |\
    ~/component/mysql/bin/mysql -N -B imagemedical| \
    sed 's/\t/ /g') 
    set -- $queryparams
    QR_HOST=$1
    QR_PORT=$2
    QR_AE=$3
    echo "Query device: $QR_HOST, $QR_PORT, $QR_AE"
fi

SCU_MYAET=""
[ -z "$AE_TITLE" ] && AE_TITLE="$DICOM_AE_TITLE"
[ -z "$STORESCU_FLAG" ] && STORESCU_FLAG="-R"
[ -n "$DEBUG" ] && STORESCU_FLAG="$STORESCU_FLAG -v -d"
[ -n "$STO" ] && STORESCU_FLAG="$STORESCU_FLAG -st $STO"
[ -n "$RTO" ] && STORESCU_FLAG="$STORESCU_FLAG -rt $RTO"
[ -n "$AE_TITLE" ] && SCU_MYAET="-aet $AE_TITLE"
[ -z "$CALL_AE" ] && CALL_AE="--call some_Store_ae"
[ -z "$FWD_HOST" ] && FWD_HOST="10.10.1.150"
[ -z "$FWD_PORT" ] && FWD_PORT="10104"
[ -z "$QR_AE" ] && QR_AE="some_Query_ae"
[ -z "$QR_HOST" ] && QR_HOST="10.10.1.150"
[ -z "$QR_PORT" ] && QR_PORT="14444"

QR_ROOT="S"
DAYS=$(date '+%Y%m%d' --date="1 days ago")-$(date '+%Y%m%d' --date="1 days ago")

#DEBUG="on"
WORKDIR=$(pwd)

# Error codes
SUCCESS=0
NEGATIVE_RESULT=1
INVALID_ARGS=2
TARGET_EXCEEDS_LOCAL=2
FILE_NOT_FOUND=3
QUERY=3
QR_FAILED=3
DATABASE_ERROR=4
SEND=4
NOT_ON_TARGET=4
DICOM_ERROR=5
LOCAL_EXCEEDS_TARGET=5
ZEROLENGTH=6
SKIPPED=7
DUPLICATE=8

   

##############################################################################
#  _setQuerycriteria_()
#
#  Set query criteria for Q/R
#  
#
#  Parameters:
#             none
#
#  Returns:
#             0 on success
#             positive number on failure
#
##############################################################################
_setQuerycriteria_() {

    echo "(0008,0052) CS [STUDY]
(0020,000d) UI [] # Study IUID
(0020,1208) IS [] # Number of Study related images
" > $WORKDIR/q.dmp

    $DCMTK_HOME/bin/dump2dcm $WORKDIR/q.dmp $WORKDIR/q.dcm >/dev/null 2>&1

    echo "(0008,0052) CS [STUDY]
" > $WORKDIR/s.dmp

    $DCMTK_HOME/bin/dump2dcm $WORKDIR/s.dmp $WORKDIR/s.dcm >/dev/null 2>&1

} # End of setQuerycriteria



##############################################################################
#  _queryStudy_()
#
#  Retrieve a study from external archive using Q/R
#  Acquire the number of study related objects that exist
#  
#
#  Parameters:
#             STYIUID - $1 - Study instance UID
#  Returns:
#             0 on success
#             positive number on failure
#
##############################################################################
_queryStudy_() {

    local styiuid=$1

    [ -f q.res ] && rm -f q.res q.ls
    # get # of objects from the archive for this study UID
    #[ -n "$DEBUG" ] && echo -n "Requesting number of images for $styiuid:"
    echo -n "$styiuid:"
    $DCMTK_HOME/bin/findscu -$QR_ROOT --aetitle $AE_TITLE --key 0020,000d="$styiuid" \
                                  --call $QR_AE $QR_HOST $QR_PORT $WORKDIR/q.dcm > q.res
    [ $? -ne 0 ] && return $QR_FAILED

    awk '
    $1=="(0020,000d)"  {s=$3;}
    $1=="(0020,1208)"  {if (s ~ "^\\[.*\\]$" ) { s= s "\t" $3; 
          gsub("\\[","",s); gsub("\\]","",s); print s; } }' q.res > q.ls

    # q.ls has styiuid and #objects - just need #objects
    if [ ! -f "q.res" ]; then
        echo "ERROR : $styiuid : Q/R failed?"
        return $QR_FAILED
    fi

    local queriedobjs=$(cat q.ls | cut -f 2)
    echo "_querystudy_: $queriedobjs"

    if [ -z "$queriedobjs" ]; then
        echo "ERROR : $styiuid : not_on_target"
        if [ "$sentobjs" -gt 0 ]; then
            return $NOT_ON_TARGET
        fi
    else
        echo "~/component/dcmtk/bin/movescu -v -d -$QR_ROOT --move $AE_TITLE --aetitle $AE_TITLE \
        --key 0020,000d=$styiuid --call $QR_AE $QR_HOST $QR_PORT s.dcm"
        read a
        ~/component/dcmtk/bin/movescu -v -d -$QR_ROOT --move $AE_TITLE --aetitle $AE_TITLE \
        --key 0020,000d=$styiuid --call $QR_AE $QR_HOST $QR_PORT s.dcm

        return $SUCCESS
    fi

} # End of queryStudy




##############################################################################
#  _retrieveStudy_()
#
#  Parameters:
#             STYIUID - $1 - Study instance UID
#             objist  - $2 - Array of objects
#  Returns:
#             0 on success
#             positive number on failure
#
##############################################################################
#
#
## This function is incomplete!
#
#
#
_retrieveStudy_() {
    # Move this to a different function
    if [ 0 ] ; then
        echo "Retrieving $styiuid from archive:"
        #$DCMTK_HOME/bin/movescu -v -d -$QR_ROOT --move $AE_TITLE --aetitle $AE_TITLE \
        $DCMTK_HOME/bin/movescu -$QR_ROOT --move $AE_TITLE --aetitle $AE_TITLE \
            --call $QR_AE $QR_HOST $QR_PORT --key 0020,000d="$styiuid" $WORKDIR/s.dcm
        [ $? -ne 0 ] && return $DICOM_ERROR

        return $SUCCESS
    fi


} # End of retrieveStudy()





## Main

[ -n "$query" ] && _setQuerycriteria_

querysuccess=$QR_FAILED
querytrycount=0
while [ "$querysuccess" -eq "$QR_FAILED" ]; do
    (( querytrycount += 1 ))
    _queryStudy_ $styiuid
    querysuccess=$?
    if [ "$querytrycount" -eq "20" ]; then
        echo "Too many query retries - Target appears down. Exiting now"
        exit 100
    fi
done

