#!/bin/sh

#
# deviceVsInst.sh
#
# gdavis - 7 November 2012
#
# Lists two columns comparing 'from' device to DICOM institution name.
#

echo "Initializing..."

# get list studies 'from' devices
zgrep StorescpReg ~/var/logs.dated/week/*info* | grep from | awk '{print $7 " "$13}' | sort | uniq > /home/medsrv/tmp/list.tmp

echo "Running..."

# get total number of studies to investigage
lineCount=`wc -l /home/medsrv/tmp/list.tmp | awk '{print $1}'`
loopCount=0

studyCount=0
mergedCount=0
invalidCount=0
invalid=0

while read study
do
	let loopCount++
        SYUI=`echo $study | awk '{print $1}' | tr -d "[]"`
        dcmDir=`~/component/tools/printStudyInfo.sh -D -s $SYUI | grep dicom | awk '{print $3}'`
        deviceName=`echo $study | awk '{print $2}'`

	# check number of files in DICOM dir excluding SCs, private objects
        count=`ls $dcmDir | grep -vE "(SC|PbT|PbR|SrT)" | wc -l`

	if [ "$dcmDir" != "" ]
	then
		let studyCount++
		
		if [ $count -gt 1 ]
		then # study has at least a few modality files, use middle file
			dcmFile=`ls $dcmDir | head -$((count/2)) | tail -1`
			category=1
		elif [ $count -eq 1 ]
		then # study has only 1 modality DICOM file, use it
			dcmFile=`ls $dcmDir`
		else # study has only SC or private objects
			# check for SC objects
			count=`ls $dcmDir | grep -vE "(PbT|PbR|SrT)" | wc -l`
			if [ $count -ge 1 ]
			then # study has a SC object
				dcmFile=`ls $dcmDir | head -$((count/2)) | tail -1`
			else # check for private objects
				count=`ls $dcmDir | wc -l`
				if [ $count -ge 1 ]
				then
					dcmFile=`ls $dcmDir | tail -1`
				else
					let invalidCount++
					invalid=1
				fi
			fi
		fi
	else
		# check for merged study
		SYUI=`zgrep $SYUI ~/var/logs.dated/week/* | grep mergestudy | sed -n 's/.*into\_\(.*\)\.[0-9]\_.*/\1/p'`
		if [ -n "$SYUI" ]
		then # study has been merged
			dcmDir=`~/component/tools/printStudyInfo.sh -D -s $SYUI | awk '{print $3}'`
			dcmFile=`ls $dcmDir | head -$((count/2)) | tail -1`
			let mergedCount++
		else # study is invalid
			let invalidCount++
			invalid=1
		fi
	fi

	if [ "$invalid" -ne "1" ]
	then
		institutionName="[`~/component/dicom/bin/dcmdump $dcmDir/$dcmFile | grep "0008,0080" | sed -n 's/.*\[\(.*\)\].*/\1/p'`]"
        	echo "$deviceName : $institutionName" >> /home/medsrv/tmp/fancylist.tmp
	else
		invalid=0
	fi

	# print progress bar
	percentage=$((loopCount*100/lineCount))
        echo -n "["
        for i in $(eval echo {0..$((percentage/2))})
        do
                echo -n "#"
        done
        for i in $(eval echo {0..$((50-percentage/2))})
        do
                echo -n " "
        done
        echo -ne "\b]"
        echo -ne " $percentage%\r"

done < "/home/medsrv/tmp/list.tmp"

echo ""

# print final results
cat /home/medsrv/tmp/fancylist.tmp | sort | uniq

echo ""
echo -e "Study Type: \t\tNormal\t\tMerged\t\tInvalid"
echo -e "# Processed\t\t$studyCount\t\t\t$mergedCount\t\t$invalidCount" 

# delete temp files
rm /home/medsrv/tmp/list.tmp
rm /home/medsrv/tmp/fancylist.tmp
