#!/bin/bash

# Find tasks matching the given criteria

# Usage: scriptname -options
# Note: dash (-) necessary

# Parse arguments
DEBUG="y"
NO_ARGS=0 
E_OPTERROR=65

if [ $# -eq "$NO_ARGS" ]  # Script invoked with no command-line args?
then
  echo "Usage: `basename $0` [-s <styiuid>]"
  exit $E_OPTERROR        # Exit and explain usage, if no argument(s) given.
fi  

while getopts "s:" Option
do
  case $Option in
    s ) SEARCH_STYIUID=1
        STYIUID="$OPTARG"
        if [ -z "$SEARCHQUEUE" ]; then
            SEARCHQUEUE="SEARCH_STYIUID"
        else
            SEARCHQUEUE="$SEARCHQUEUE SEARCH_STYIUID"
        fi
    ;;

    * ) echo "Unimplemented option chosen."
        exit $E_OPTERROR
    ;;   # DEFAULT
  esac
done

if [ -n "$DEBUG" ]; then
    echo;echo;echo
    echo "*********  DEBUG  **********"
    echo "SEARCH_STYIUID=$SEARCH_STYIUID, STYIUID=$STYIUID"
    echo "SEARCHQUEUE=$SEARCHQUEUE"
    echo "*********  DEBUG  **********"
    echo;echo;echo
fi

shift $(($OPTIND - 1))
#  Decrements the argument pointer so it points to next argument.
#  $1 now references the first non option item supplied on the command line
#+ if one exists.

unset SEARCHRESULTS

for search in "$SEARCHQUEUE"; do

  case "$search" in
    SEARCH_STYIUID )
                     echo "Searching for tasks matching STYIUID $STYIUID"
                     SEARCHRESULTS=$(find ~/var/{cpu,net}queue -type f 2>/dev/null|xargs grep -l "$STYIUID" 2>/dev/null)
    ;;

    * )
       echo "Bad search type (shouldn't ever get here)"
       exit $BAD_SEARCH
    ;;
  esac
done

if [ -n "$SEARCHRESULTS" ]; then
   echo "Results found:"
   echo -e "$SEARCHRESULTS"|sed 's/ /\n/g'
else
   echo "No results found"
fi

exit 0

