#!/bin/bash
#
# fixLegacyConflict.sh - if checkLegacyStudies.sh finds that a study directory exists in both ~/data/dicom and ~/data/dicom.hashed
#                        it can not correct them.  This script will move the data from dicom to dicom.hashed and created a link
#

FIN="$1"



[ -z "$FIN" ] && echo -e "\n\n\tERROR:  Missing required parameter STUDYLISTFILE\n\n" && exit

for s in `cat $FIN`
do
   if [ "$s" = "" ] ; then
      continue
   fi

   echo -e "\n$s"
   hashdir=`$HOME/component/utils/bin/getHashDirectory -i $s`
   OrigStudyDir="$HOME/data/dicom/$s"
   HashStudyDir="$HOME/data/dicom.hashed/$hashdir"
   if [ -d "$OrigStudyDir" ] ; then
      if [ -d "$HashStudyDir" ] ; then
         echo -e "\t$s has both dicom and dicom.hashed directories, correcting"
         echo -e "\tmv $OrigStudyDir/* $HashStudyDir"
         mv $OrigStudyDir/* $HashStudyDir
         rc=$?
         if [ "$rc" = "0" ] ; then
             echo -e "\tmv successful.  Cleaning up $s and linking"
             echo -e "\trm -rf $OrigStudyDir"
             rm -rf $OrigStudyDir/
             rmrc=$?
             if [ "$rc" = "0" ] ; then
                  echo -e "\trm successful, linking."
                  echo -e "\tln -s $HashStudyDir $OrigStudyDir"
                  ln -s $HashStudyDir $OrigStudyDir
             else
                  echo -e "\trm failed.  Returned $rc.  Skipping"
                  echo "$s" >> FailedLegacyConflictfix.stys
             fi
         else
             echo -e "\tmv failed.  Returned $rc.  Skipping"
             echo "$s" >> FailedLegacyConflictfix.stys
         fi
      else
         echo -e "\t$s does not have a dicom.hashed directory: $HashStudyDir.  Skipping...."
         continue
      fi
   else
      echo -e "\t$s does not have a study directory: $OrigStudyDir.  Skipping...."
      continue
   fi
done 
