#!/bin/sh
#
# Description:
#    Finds and fixes silly user lists.
#
# Usage:
#   For usage see usage_()
#
# Last Updated     : $Author: tnemeth $
# Update Date Time : $Date: 2007/10/30 16:17:39 $
# RCS File Name    : $RCSfile: exportusers.sh,v $
# RCS Revision     : $Revision: 1.2 $
# Symbolic name    : $Name:  $
# State            : $State: Exp $
#

. $HOME/etc/setenv.sh

###############################################################################
# cRefr="30"
# cPageSize="150"
###############################################################################

###############################################################################
# SETTING DEFAULTS
###############################################################################
now=`date +%Y-%m-%d-%h-%s`
UserList="${TMP}/$$.userlist.$$"
ls ${USERS_DATA} -1 | grep -v "^#" > "$UserList"
OutputFile="sillyUserLists.Found.$now"
OutputDir="$HOME/work/SillyUserLists"
RefreshTime=300
PageSize=50

if [ ! -e $OutputDir ] ; then
	echo -e "\n\nOutput Dir: $OutputDir does not exist making it..."
	mkdir -p $OutputDir
fi
###############################################################################
# FUNCTIONS
###############################################################################

#
# Prints usage information
#
# $1 - script name
#
function usage_() {
	cat <<-EOFUSAGE
		Usage: $1 [ -F ] [ -o <output file> ] [ -u <username> ] [ -f <userlist file> ] [ -r <RefreshTime> ] [ -s <PageSize> ] [ -h ] [ -l ]
		   -F                    Also correct the silly user's list, not just identify them
		   -o <file>             Name of output file.  Default is $OutputFile
		   -f <file>             Operate on users listed in file.
		   -h                    Display help
		   -l                    List users on the system
		   -u <user>             Only find/fix lists for user.
		   -r <RefreshTime>      Set the threshold for Page Refresh Time.  Default is: $RefreshTime.
		   -s <PageSize>         Set the threshold for Page Size.  Default is: $PageSize.
	EOFUSAGE

	exit 0
}

#
# Lists users on the system
#
function listUsers_ () {
	echo -e "\n\nUsers on the system:\n"
	ls ${USERS_DATA} -1 | grep -v "^#"
	echo -e "\n\n"
	exit 0
} # End function listUsers


###############################################################################
# ARGUMENT PARSING
###############################################################################
while getopts "Fo:f:hlu:r:s:" OPT ; do
	case $OPT in
		F) FIX="TRUE" ;;
		o) OutputFile="$OPTARG" ;;
		f) UserList="$OPTARG" ;;
		h) usage_ `basename $0` ;;
		l) listUsers_ ;;

		u) User="$OPTARG"
		SPECIFICUSERONLY="TRUE"
		;;

		r) RefreshTime="$OPTARG"
		;;

		s) PageSize="$OPTARG"
		;;

		*) echo "No such option $OPTARG"
		usage_ `basename $0`
		;;

	esac
done


###############################################################################
# MAIN
###############################################################################

#echo -e  "\n\tList of Offending Worklist filter files, if any, will be saved to: $OutputFile"
#if [ "$FIX" = "TRUE" ] ; then
#	echo -e "\n\nCaution:  You are running the script in FIX MODE!  User Lists will be modified.  Copies of the original lists"
#	echo -e "will be saved to $HOME/work/SillyUserLists/ directory in a subdirectory named after the users id, "
#	echo -e "i.e. If the user is drhoward copies of the original lists would be saved to $HOME/work/SillyUserLists/drhoward/\n\n"
#	echo -e "Are you sure you want to do this?  [ y / n ]\n\n"
#	read answer
#	if [ "$answer" != "y" ] ; then
#		echo -e "\nYou answered: $answer EXITING..."
#		exit 1
#	fi
#fi
#sleep 10

if [ "$SPECIFICUSERONLY" = "TRUE" ] ; then
	echo "$User" > $UserList
fi


DATE=`date +"%Y%m%d%H%M%S"`
PID=$$
COUNTER=0

OIFS="$IFS"
IFS="
"
for u in `cat $UserList` ; do
	NUMFILTERS=`ls ${USERS_DATA}/$u/filters/ | wc -l`
	if [ $NUMFILTERS -eq 0 ] ; then
		#echo "$u has no filters set"
		continue
	fi

	echo "Current User: $u"
	for f in `ls -1 ${USERS_DATA}/$u/filters/*.rec 2>/dev/null` ; do
		. ${f}

#		fixed=""
#		if [ $cRefr -gt 0 -a $cRefr -lt $RefreshTime ] ; then
#			echo "User: $u : Filter: $f : Refresh: $cRefr" >> $OutputFile
#			if [ "$FIX" = "TRUE" ] ; then
#				mkdir -p $HOME/work/SillyUserLists/$u/filters/
#				cp ${f} $HOME/work/SillyUserLists/$u/filters/`basename ${f}`.ORIG.refresh
#				cat ${f} | grep -v cRefr > ${f}.new
#				echo -e "\ncRefr=\"300\"\n" >> ${f}.new
#				mv ${f}.new ${f}
#				fixed="true"
#			fi
#		fi
#		if [ $cPageSize -gt $PageSize ] ; then
#			echo "User: $u : Filter: $f : PageSize: $cPageSize" >> $OutputFile
#			if [ "$FIX" = "TRUE" ] ; then
#				mkdir -p $HOME/work/SillyUserLists/$u/filters/
#				cp ${f} $HOME/work/SillyUserLists/$u/filters/`basename ${f}`.ORIG.pagesize
#				cat ${f} | grep -v PageSize > ${f}.new
#				echo -e "\ncPageSize=\"20\"\n" >> ${f}.new
#				mv ${f}.new ${f}
#				fixed="true"
#			fi
#		fi
		
		mkdir -p $HOME/work/SillyUserLists/$u/filters/
		cp ${f} $HOME/work/SillyUserLists/$u/filters/`basename ${f}`.ORIG.pagesize
		cat ${f} | grep -v PageSize > ${f}.new
		echo -e "\ncPageSize=\"$PageSize\"\n" >> ${f}.new
		mv ${f}.new ${f}
		fixed="true"

		if [ "$fixed" = "true" ] ; then
			let COUNTER++;
			PROLIFERATE_TMP="$TMP/0107.$DATE.$PID.$COUNTER"
			echo "${DOTCOM_HOME}/proliferate.sh -f \"$f\"" >${PROLIFERATE_TMP}
			chown $USER.$USER ${PROLIFERATE_TMP}
			mv ${PROLIFERATE_TMP} ${TASKD_NET_QUEUE}
		fi	
	done
done
IFS="$OIFS"
