#!/bin/bash
#
#  $1 - archive mount point to make room on, i.e. a2

# Source needed files...
. $HOME/var/arch/archive.cfg

# Setup static variables...
AROOT="gmas1"
ARCHBASE="$HOME/var/archive"
DB="$HOME/component/mysql/bin/mysql -N imagemedical"

#
# Functions
#
function usage {
  echo -e "\n\n$0 STYLISTFILE "
  echo -e "\nWHERE:\n+ STYLISTFILE is a file containing studies to be moved"
  echo -e "\n\n"
}

# Check for input file 
if [ -z "$1" ] ; then
  usage
  exit 120
elif [ ! -e "$1" ] ; then
  echo -e "\n\nInput file, $1 , does not exist.  Exiting\n\n"
  exit
fi

# Check to make sure we are doing what we want to do
echo -e "\n\nArchive Parameters:"
echo -e "\tArchive: $Archive"
echo -e "\tMountPoint: $MountPoint"
echo -e "\tAROOT: $AROOT"
echo -e "\n\tThis script will move studies listed in $1 from $AROOT to $MountPoint."
echo -e "\tDO YOU WANT TO CONTINUE? (y/n)"

read answer
echo -e "\n\nYou answered $answer\n"

if [ "$answer" != "y" ] ; then
   echo -e "\n\n$answer does not equal y exiting\n\n"
   exit
fi

STYLIST="$1"
count=1

echo -e "Input file, STYLIST: $STYLIST"
echo -e "Counter: $count"

# Loop through list and do the work...
for s in `cat $STYLIST`
do
   echo -e "\n\n---------------------------------------------------------"
   ARCHDIR=`echo "select ARCHDIR from Archive where STYIUID = '$s'" | $DB`
   smartstore=`echo $ARCHDIR | grep -i ${MountPoint}` # already in gmas1/SmartStore

   # If the study is already in the current mount point directory, then skip it
   if [ ! -z "$smartstore" ] ; then
      continue
   fi

   NEWARCHDIR=`echo $ARCHDIR | sed -e "s!$AROOT!$MountPoint!"`
   echo -e "\t$count:\t$s\t$ARCHDIR\t$NEWARCHDIR"
   if [ ! -e $ARCHBASE/$NEWARCHDIR/ ] ; then
      mkdir -p $ARCHBASE/$NEWARCHDIR
      RES=$?
      if [ $RES -ne 0 ] ; then
         echo "mkdir -p $ARCHBASE/$NEWARCHDIR returned $RES, skipping"
         continue
      fi
   fi

   cp -va $ARCHBASE/$ARCHDIR/$s $ARCHBASE/$NEWARCHDIR/
   RES=$?

   if [ $RES -eq 0 ] ; then
      echo "update Archive set ARCHDIR = '$NEWARCHDIR' where STYIUID = '$s'" | $DB
      echo "Updating DB, setting ARCHDIR = $NEWARCHDIR where STYIUID = $s"
      if [ ! -z "$s" ] ; then
         echo "Removing old archive directory: $ARCHBASE/$ARCHDIR/$s/"
         rm -rf $ARCHBASE/$ARCHDIR/$s/
      fi
   else
      echo "cp -va $ARCHBASE/$ARCHDIR/$s $ARCHBASE/$NEWARCHDIR/ returned $RES.  Not updating DB"
      continue
   fi

   # Richardson cleanup only
   if [ -e $ARCHBASE/$NEWARCHDIR/$s/.info/obsoleted.rec ] ; then
       echo "Clearing obsoleted.rec file"
       rm -f $ARCHBASE/$NEWARCHDIR/$s/.info/obsoleted.rec
   fi
   # End Richardson cleanup only
   count=$((count+1)) 
   echo -e "\nsleeping 15 secs\n"
   sleep 15
done

