#! /bin/bash

# Given a CSV file containing user information, add users via the http api

# Must have the following configured in var/iface/access.cfg:
# 127.0.0.1="A"

# Company,User ID,Last,First,MI,Address,Address,Address,Address,Address,PhoneNumber1,PhoneNumber2,Note
#cat $1 | awk -F "," '{ print $1 " " $2 " " $3 " " $4 " " $5 " " $6 " " $7 " " $8 " " $9 " " $10 " " $11 " " $12 " " $13 }'| \

csv=$1
del=$2
#sleeptime=1

# Are we deleting or adding/modifying?
[ -z "$del" ] && cOper=change || cOper=delete

fields0="company ID last first mi a1 a2 a3 a4 a5 PHONE1 PHONE2 Note"
fields1="ID ADDRESS PHONE1 PHONE2 Note USERNAME PASSWD RIGHTS AGROUP"
fields2="cOper c_ID c_ADDRESS c_PHONE1 c_PHONE2 c_Note c_USERNAME c_PASSWD c_RIGHTS c_AGROUP"
OIFS=$IFS
READIFS=","
IDLIST=""
MAXIDCHARS=12
_fixIDMAXIDCHARS=11


function _urlEncode() {
    # usage exit for too many parameters
    [ $# -gt 1 ] && { echo >&2 "usage: _urlEncode [string]"; return 1; }
    
    # self call when an argument is given, else handle stdin
    [ $# -eq 1 ] && { echo -n "$1" | _urlEncode; return $?; }
        
    # first create alternating lines of hex code and ascii characters
    # then remember the hex value,
    # convert spaces to +,
    # keep some selected characters unchanged
    # and use the hexvalue prefixed with a % for the rest
    ##od -t x1c -w1 -v -An |
    ##LANG=C awk '
    ##  NR % 2                  { hex=$1;               next    }
    ##  /^ *$/                  { printf("%s", "+");    next    }
    ##  /^ *[a-zA-Z0-9.*()-]$/  { printf("%s", $1);     next    }
    ##  /^ *\//                 { printf("%s", $1);     next    }
    ##                          { printf("%%%s", hex)           }
    ##'
    hexdump -v -e '1/1 "%02x\t"' -e '1/1 "%_c\n"' |
    LANG=C awk '
        $1 == "20"                      { printf("%s",      "+");   next    }
        $2 ~  /^[a-zA-Z0-9.*()\/-]$/    { printf("%s",      $2);    next    }
                                        { printf("%%%s",    $1)             }
    '
}

function _fixID() {
    # $1 : ID to check
    # $2 : (optional) count found so far
    local _id=$1
    [ -n "$2" ] && local _count=$2 || local _count=0
    local _existingid
    local _matched
    local _newid

       
    for _existingid in $IDLIST; do
        if [ "$_id" == "$_existingid" ]; then
           _matched=1
           break
        fi
    done

    if [ -n "$_matched" ]; then
        (( _count += 1 ))
        if [ "$_count" -gt 9 ]; then
            exit 100 # 
        fi
        if [ "${#_id}" -gt "$_fixIDMAXIDCHARS" ]; then
            _id=${_id:0:"$_fixIDMAXIDCHARS"} 
        fi
        _newid=${_id}${_count}
        _fixID $_newid $_count
    else
        echo "$_id"
    fi
}        

         
    
IFS=$READIFS
# Read in values from CSV
cat $csv | \
while read company ID last first mi a1 a2 a3 a4 a5 PHONE1 PHONE2 Note; do
    IFS=$OIFS
    # Strip trailing spaces
    for myvar in $fields0; do
        t=$(echo ${!myvar}|sed 's/ \{1,\}$//g') #strip trailing spaces
        export "${myvar}"="${t}"
#        echo "$myvar : ${!myvar}"
    done

    # Deal with userid > 12 characters
    # I do iT here because _fixID is recursive, and it seems stupid to check the userid length
    # during every recursive call.
    if [ "${#ID}" -gt "$MAXIDCHARS" ]; then
        ID=${ID:0:$MAXIDCHARS}
    fi

    # Handle duplicate userids (man, python's list datatype would be handy here!)
    ID=$(_fixID $ID 0)
    [ -n "$IDLIST" ] && IDLIST="$IDLIST $ID" || IDLIST=$ID

    # Populate values not in qsv below
    USERNAME=$last^$first^$mi^$pre^$suf
    ADDRESS="$a1 $a2 $a3 $a4 $a5"
    PASSWD=$ID
    RIGHTS="O"
    AGROUP="Physician"

    # URL encode
    for myvar in $fields1; do
        export "c_${myvar}"=$(_urlEncode "${!myvar}")
    done

    # Echo field values (debugging)
    echo "----------------------------------"
    for myvar in $fields2; do
        echo "$myvar : ${!myvar}"
        echo -n "${!myvar}," >> ${csv}.added.$$
        echo >> ${csv}.added.$$
    done

# Call curl with correct parameters
# fields2="cOper c_ID c_ADDRESS c_PHONE1 c_PHONE2 c_Notes c_USERNAME c_PASSWD c_RIGHTS c_AGROUP"
#curl "http://127.0.0.1/iface/userOper.jsp?cOper=change&c_ID=autotest&c_PASSWD=test&c_RIGHTS=A&c_AGROUP=Admin&c_USERNAME=Riley^Charles^W%2E&c_ADDRESS=9+Pilgrim+Rd%2E%0D%0AGreenville%2C+SC%2E%0D%0A29607"
    echo Calling curl: curl "http://127.0.0.1/iface/userOper.jsp?cOper=$cOper&c_ID=$c_ID&c_PASSWD=$c_PASSWD&c_RIGHTS=$c_RIGHTS&c_AGROUP=$c_AGROUP&c_USERNAME=$c_USERNAME&c_ADDRESS=$c_ADDRESS&c_PHONE1=$c_PHONE1&c_PHONE2=$c_PHONE2&c_Note=$c_Note"

    curl "http://127.0.0.1/iface/userOper.jsp?cOper=$cOper&c_ID=$c_ID&c_PASSWD=$c_PASSWD&c_RIGHTS=$c_RIGHTS&c_AGROUP=$c_AGROUP&c_USERNAME=$c_USERNAME&c_ADDRESS=$c_ADDRESS&c_PHONE1=$c_PHONE1&c_PHONE2=$c_PHONE2&c_Note=$c_Note"
    retval=$?
    echo "curl returns: $retval"

    [ -n "$sleeptime" ] && sleep $sleeptime

    IFS=$READIFS

done

IFS=$OIFS

exit 0
