#! /bin/bash

#debug=1
medsrv_version=$1
MEDUSR="medsrv"
OTHERSERVERS=""
OTHERSERVERSFILE="otherservers"
repository_dir="/home/$MEDUSR/patch"
utils="/home/medsrv/applyupdates.sh /home/medsrv/impatch"
MYSQL_HOME="/home/medsrv/component/mysql"
# Query info about parent or child servers
PARENT=`su - $MEDUSR -c "$MYSQL_HOME/bin/mysql -N -B -e 'select HOST, TYPE from Target;' imagemedical|awk -F'\t' '{ if ( \\$2 == \"P\" ) print \\$1; }'"`
CHILDREN=`su - $MEDUSR -c "$MYSQL_HOME/bin/mysql -N -B -e 'select HOST, TYPE from Target;' imagemedical|awk -F'\t' '{ if ( \\$2 == \"C\" ) print \\$1; }'"`

[ -f "$OTHERSERVERSFILE" ] && . $OTHERSERVERSFILE

serverlist=$(echo -n $PARENT; echo -n " "; echo -n $CHILDREN; echo -n " "; echo -n $OTHERSERVERS)

[ -n "$PARENT" ] && echo "Info: Parent server(s) found: $PARENT."

[ -n "$CHILDREN" ] && echo "Info: Child server(s) found: `for e in $CHILDREN; do echo -n [$e] ;done`."

[ -n "$OTHERSERVERS" ] && echo "Info: Other server(s) found: `for e in $OTHERSERVERS; do echo -n [$e] ;done`."

[ -z "$medsrv_version" ] && medsrv_version=$(head -1 /home/medsrv/etc/component.ls \
                                            |sed 's/from helen.eradimagemedical.com, Version //' \
                                            | cut -d . -f -2)
if [  -z "$medsrv_version" ]; then
    echo "Can't determine installed version and no version specified."
    echo "Don't know what to do!"
    exit 1
fi

patchdir_version=$(echo $medsrv_version |sed 's/\.//')

[ -n "$serverlist" -a -n "$debug" ] && echo "Servers to update: $serverlist"    
echo "Press enter to continue synchronizing all version $medsrv_version patches"
echo "to this list of targets"
echo "or hit Control-C to abort."
read a

for target in $serverlist; do
    rsync -va -e ssh --progress --delete $repository_dir/patch.$patchdir_version $target:$repository_dir
    if [ $? -ne 0 ]; then
        echo "rsync returned with error $?"
        echo "Problem on host $target"
        problemlist=$(echo $problemlist $target)
    else
        echo
        echo "Patch directory $repository_dir/patch.$patchdir_version has been synchronized on host $target."
        echo
    fi
    echo "Syncing impatch and applyupdates:"
    for file in $utils; do
        rsync -va -e ssh --progress $file $target:/home/medsrv/
    done
done

if [ -n "$problemlist" ]; then
    echo "ALERT!"
    echo "Problems occurred while synchronizing the following servers:"
    echo -e $problemlist|sed 's/ /\n/g'
    echo
    echo "Check that the server's IP is correct in the device table and that"
    echo "the target is pingable"
    echo
    echo "Patch sychronisation completed with errors."
fi

