#! /bin/bash
# ex: set expandtab shiftwidth=4 softtabstop=4 syntax=sh:

if [ "$UID" -ne 0 ]; then
    echo "Not Root"
    echo "This script must be run as root"
    exit 1
fi

# action can be "backup", "restore", or "prune"
# backuptype can be "os", "medsrv", "users", or "logs"
# if backuptype is not specified and action is "backup", do all four

action=$1
backuptype=$2

myhostname=$(hostname -s)

num_logs_to_keep=1
num_medsrv_to_keep=3
num_os_to_keep=1
num_users_to_keep=5

backupdir=/home/configbackup
unset backuphosts
# define backuphosts in sbconfigfile to back up to one or more remote hosts

logsbackupfilebasename=medsrv_log_backup
medsrvbackupfilebasename=medsrv_config_backup
usersbackupfilebasename=medsrv_user_backup
osbackupfilebasename=os_config_backup

# File lists
osfilestobackup="/etc/cups /etc/hosts /etc/modules.conf /etc/resolv.conf /etc/fstab $(ls /etc/sysconfig/network-scripts/route-*) $(ls /etc/sysconfig/network-scripts/ifcfg-*) /etc/sysconfig/network /tmp/dfout /tmp/fdiskout /etc/exports /etc/iscsi.conf /etc/multipath.conf /etc/cluster /home/image/cluster /home/image/tools/serverbackup/serverbackup.cfg.$(hostname) /home/image/tools/serverbackup/serverbackup.sh /etc/rc.local /etc/initiatorname.iscsi"

medsrvfilestobackup="var/dcmtk/printer.cfg var/conf var/arch var/openssl var/dicom var/prior var/iface var/hl7 date/layout_template data/attachment data/layout_template data/main data/users/\@system data/report_template date/calculated_fields tmp/target.dump tmp/devsrv.dump tmp/archive.dump component/apache/conf.tpl tmp/version_at_time_of_backup var/conf/reportsaving.cfg tmp/SchedulerCombinedProcedure.dump tmp/SchedulerDevice.dump tmp/SchedulerDeviceProcedure.dump tmp/SchedulerFacility.dump tmp/SchedulerGeneratedNumbers.dump tmp/SchedulerICD9.dump tmp/SchedulerInsurer.dump tmp/SchedulerLog.dump tmp/SchedulerOrder.dump tmp/SchedulerOrderInsurer.dump tmp/SchedulerOrderReferrer.dump tmp/SchedulerPatient.dump tmp/SchedulerPatientInsurer.dump tmp/SchedulerProcedure.dump tmp/SchedulerProcedureICD9.dump tmp/SchedulerProcedureInterRelation.dump tmp/SchedulerReferrer.dump etc/virthost.sh"

logfilestobackup="var/logs.dated"

# Override any of the config variables above in $sbconfigfile
sbconfigfile=/home/image/tools/serverbackup/serverbackup.cfg.$(hostname)

# Functions ----------

function usage() {
        echo "Usage: $0 backup [os|medsrv|users|logs]"
        echo
        echo "If the second argument is not specified, the script backs up all four"
        echo
        exit 1
}

function pruneBackup() {
    # btype can be "os", "medsrv", "users", or "logs"
    # Prunes backups to specified number on local server and remote hosts defined in $backuphosts
    local btype=$1
    local count=0
    local what_to_prune
    local numtokeep

    case $btype in
        os) what_to_prune="${myhostname}.${osbackupfilebasename}.*"
            numtokeep=$num_os_to_keep
        ;;
        medsrv) what_to_prune="${myhostname}.${medsrvbackupfilebasename}.*"
                numtokeep=$num_medsrv_to_keep
        ;;
        users) what_to_prune="${myhostname}.${usersbackupfilebasename}.*"
               numtokeep=$num_users_to_keep
        ;;
        logs) what_to_prune="${myhostname}.${logsbackupfilebasename}.*"
              numtokeep=$num_logs_to_keep
        ;;

        *) return ;;
    esac
        
    for f in $(ls -t ${backupdir}/${what_to_prune}); do
        (( count += 1 ))
        if [ $count -gt $numtokeep ]; then
            rm -f $f
            echo "Removing old backup: $f on $myhostname"
            if [ -n "$backuphosts" ]; then
                for h in $backuphosts; do
                    echo "Removing old backup: $f on $h"
                    ssh root@$h "rm -f $f"
                done
            fi
        fi
    done

} # pruneBackup()

function doBackup() {
    # btype can be "os", "medsrv", "users", or "logs"
    local btype=$1
    unset filelist

    case $btype in
        os) filelist="${osfilestobackup}"
            backupfile="${backupdir}/${myhostname}.${osbackupfilebasename}.$(date +%s).tar"
            df -h > /tmp/dfout
            fdisk -l > /tmp/fdiskout
            tar -cvf $backupfile $filelist 2>/dev/null
            echo "Cleaning up /tmp"
            rm -f /tmp/dfout /tmp/fdiskout
        ;;
        medsrv) filelist="${medsrvfilestobackup}"
                backupfile="${backupdir}/${myhostname}.${medsrvbackupfilebasename}.$(date +%s).tar"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical Target > tmp/target.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical Archive > tmp/archive.dump 2>/dev/null"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical DevServ > tmp/devsrv.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerCombinedProcedure > tmp/SchedulerCombinedProcedure.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerDevice > tmp/SchedulerDevice.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerDeviceProcedure > tmp/SchedulerDeviceProcedure.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerFacility > tmp/SchedulerFacility.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerGeneratedNumbers > tmp/SchedulerGeneratedNumbers.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerICD9 > tmp/SchedulerICD9.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerInsurer > tmp/SchedulerInsurer.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerLog > tmp/SchedulerLog.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerOrder > tmp/SchedulerOrder.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerOrderInsurer > tmp/SchedulerOrderInsurer.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerOrderReferrer > tmp/SchedulerOrderReferrer.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerPatient > tmp/SchedulerPatient.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerPatientInsurer > tmp/SchedulerPatientInsurer.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerProcedure > tmp/SchedulerProcedure.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerProcedureICD9 > tmp/SchedulerProcedureICD9.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerProcedureInterRelation > tmp/SchedulerProcedureInterRelation.dump"
                su - medsrv -c "component/mysql/bin/mysqldump imagemedical SchedulerReferrer > tmp/SchedulerReferrer.dump"
                su - medsrv -c "head -1 etc/component.ls > tmp/version_at_time_of_backup"
                su - medsrv -c "./applyupdates.sh -q >> tmp/version_at_time_of_backup"
                su - medsrv -c "tar -cvf $backupfile $filelist 2>/dev/null"
        ;;
        users)  backupfile="${backupdir}/${myhostname}.${usersbackupfilebasename}.$(date +%s).tar"
                su - medsrv -c "component/tools/exportusers.sh -f $backupfile"
        ;;
        logs)   filelist="${logfilestobackup}"
                backupfile="${backupdir}/${myhostname}.${logsbackupfilebasename}.$(date +%s).tar"
                su - medsrv -c "tar -cvf $backupfile $filelist 2>/dev/null"
        ;;

        *) return ;;
    esac

    if [ -n "$backuphosts" ]; then
        for h in $backuphosts; do
            echo "Copying $backupfile to $h:$backupdir/"
            rsync -va -e ssh $backupfile $h:$backupdir/ --progress
        done
    fi
} # doBackup()


# Main ----------

[ -z "$action" ] && usage

[ -e "$sbconfigfile" ] &&  . $sbconfigfile

if [ ! -d "$backupdir" ]; then
    mkdir -p $backupdir
fi

chown medsrv.medsrv $backupdir

if [ -n "$backuphosts" ]; then
    for h in $backuphosts; do
        ssh root@$h "if [ ! -d \"$backupdir\" ]; then
                                  mkdir -p $backupdir;
                                  chown medsrv.medsrv $backupdir;
                              fi"
    done
fi
     
case $action in
    restore)    echo "restore not yet implemented"
    ;;
    backup)     if [ -z "$backuptype" ]; then
                    for t in os medsrv users logs; do
                        doBackup $t
                        pruneBackup $t
                    done
                else
                    doBackup $backuptype
                    pruneBackup $backuptype
                fi
    ;;
    *)          usage
    ;;
esac
